@@ -2,7 +2,7 @@
 # This file is part of pygal
 #
 # A python svg graph plotting library
-# Copyright © 2012-2015 Kozea
+# Copyright © 2012-2016 Kozea
 #
 # This library is free software: you can redistribute it and/or modify it under
 # the terms of the GNU Lesser General Public License as published by the Free
@@ -20,18 +20,15 @@
 
 from __future__ import division
 
-from itertools import chain, repeat
-
 from math import ceil, cos, sin, sqrt
 
+from pygal import stats
 from pygal._compat import is_list_like, is_str, to_str
 from pygal.graph.public import PublicApi
 from pygal.interpolate import INTERPOLATIONS
-from pygal import stats
 from pygal.util import (
-    cached_property, compute_scale, cut, decorate,
-    get_text_box, get_texts_box, majorize, rad, reverse_text_len,
-    split_title, truncate, filter_kwargs)
+    cached_property, compute_scale, cut, decorate, filter_kwargs, get_text_box,
+    get_texts_box, majorize, rad, reverse_text_len, split_title, truncate)
 from pygal.view import LogView, ReverseView, View, XYLogView
 
 
@@ -232,7 +229,9 @@ class Graph(PublicApi):
         if not self._y_labels or not self.show_y_labels:
             return
 
-        axis = self.svg.node(self.nodes['plot'], class_="axis y")
+        axis = self.svg.node(self.nodes['plot'], class_="axis y%s" % (
+            ' always_show' if self.show_y_guides else ''
+        ))
 
         if (0 not in [label[1] for label in self._y_labels] and
                 self.show_y_guides):
@@ -346,20 +345,8 @@ class Graph(PublicApi):
         h = max(self.legend_box_size, self.style.legend_font_size)
         x_step = self.view.width / cols
         if self.legend_at_bottom:
-            # if legends at the bottom, we dont split the windows
-            # gen structure - (i, (j, (l, tf)))
-            # i - global serie number - used for coloring and identification
-            # j - position within current legend box
-            # l - label
-            # tf - whether it is secondary label
-            gen = enumerate(enumerate(chain(
-                zip(self._legends, repeat(False)),
-                zip(self._secondary_legends, repeat(True)))))
             secondary_legends = legends  # svg node is the same
         else:
-            gen = enumerate(chain(
-                enumerate(zip(self._legends, repeat(False))),
-                enumerate(zip(self._secondary_legends, repeat(True)))))
 
             # draw secondary axis on right
             x = self.margin_box.left + self.view.width + self.spacing
@@ -375,44 +362,56 @@ class Graph(PublicApi):
                 self.nodes['graph'], class_='legends',
                 transform='translate(%d, %d)' % (x, y))
 
-        for (global_serie_number, (i, (title, is_secondary))) in gen:
+        serie_number = -1
+        i = 0
 
-            col = i % cols
-            row = i // cols
+        for titles, is_secondary in (
+                (self._legends, False),
+                (self._secondary_legends, True)):
+            if not self.legend_at_bottom and is_secondary:
+                i = 0
 
-            legend = self.svg.node(
-                secondary_legends if is_secondary else legends,
-                class_='legend reactive activate-serie',
-                id="activate-serie-%d" % global_serie_number)
-            self.svg.node(
-                legend, 'rect',
-                x=col * x_step,
-                y=1.5 * row * h + (
-                    self.style.legend_font_size - self.legend_box_size
-                    if self.style.legend_font_size > self.legend_box_size
-                    else 0
-                ) / 2,
-                width=self.legend_box_size,
-                height=self.legend_box_size,
-                class_="color-%d reactive" % (
-                    global_serie_number % len(self.style.colors))
-            )
+            for title in titles:
+                serie_number += 1
+                if title is None:
+                    continue
+                col = i % cols
+                row = i // cols
 
-            if isinstance(title, dict):
-                node = decorate(self.svg, legend, title)
-                title = title['title']
-            else:
-                node = legend
+                legend = self.svg.node(
+                    secondary_legends if is_secondary else legends,
+                    class_='legend reactive activate-serie',
+                    id="activate-serie-%d" % serie_number)
+                self.svg.node(
+                    legend, 'rect',
+                    x=col * x_step,
+                    y=1.5 * row * h + (
+                        self.style.legend_font_size - self.legend_box_size
+                        if self.style.legend_font_size > self.legend_box_size
+                        else 0
+                    ) / 2,
+                    width=self.legend_box_size,
+                    height=self.legend_box_size,
+                    class_="color-%d reactive" % serie_number
+                )
 
-            truncated = truncate(title, truncation)
-            self.svg.node(
-                node, 'text',
-                x=col * x_step + self.legend_box_size + 5,
-                y=1.5 * row * h + .5 * h + .3 * self.style.legend_font_size
-            ).text = truncated
+                if isinstance(title, dict):
+                    node = decorate(self.svg, legend, title)
+                    title = title['title']
+                else:
+                    node = legend
 
-            if truncated != title:
-                self.svg.node(legend, 'title').text = title
+                truncated = truncate(title, truncation)
+                self.svg.node(
+                    node, 'text',
+                    x=col * x_step + self.legend_box_size + 5,
+                    y=1.5 * row * h + .5 * h + .3 * self.style.legend_font_size
+                ).text = truncated
+
+                if truncated != title:
+                    self.svg.node(legend, 'title').text = title
+
+                i += 1
 
     def _make_title(self):
         """Make the title"""
@@ -643,7 +642,7 @@ class Graph(PublicApi):
                     map(lambda x: truncate(x, self.truncate_legend or 15),
                         [serie.title['title']
                          if isinstance(serie.title, dict)
-                         else serie.title for serie in series_group]),
+                         else serie.title or '' for serie in series_group]),
                     self.style.legend_font_size)
                 if self.legend_at_bottom:
                     h_max = max(h, self.legend_box_size)
@@ -730,9 +729,9 @@ class Graph(PublicApi):
         if self.print_values_position == 'top':
             gh = self.height - self.margin_box.y
             alpha = 1.1 * (self.style.value_font_size / gh) * self._box.height
-            if self._max > 0:
+            if self._max and self._max > 0:
                 self._box.ymax += alpha
-            if self._min < 0:
+            if self._min and self._min < 0:
                 self._box.ymin -= alpha
 
     def _confidence_interval(self, node, x, y, value, metadata):
@@ -894,8 +893,8 @@ class Graph(PublicApi):
             self._y_labels_major = []
 
     def add_squares(self, squares):
-        x_lines = squares[0]-1
-        y_lines = squares[1]-1
+        x_lines = squares[0] - 1
+        y_lines = squares[1] - 1
 
         _current_x = 0
         _current_y = 0
@@ -906,7 +905,8 @@ class Graph(PublicApi):
                 self.nodes['plot'], 'path',
                 class_='bg-lines',
                 d='M%s %s L%s %s' % (
-                    _current_x, 0, _current_x, self.height-self.margin_box.y))
+                    _current_x, 0, _current_x,
+                    self.height - self.margin_box.y))
 
         for line in range(y_lines):
             _current_y += (self.height - self.margin_box.y) / squares[1]
@@ -914,7 +914,7 @@ class Graph(PublicApi):
                 self.nodes['plot'], 'path',
                 class_='bg-lines',
                 d='M%s %s L%s %s' % (
-                    0, _current_y, self.width-self.margin_box.x, _current_y))
+                    0, _current_y, self.width - self.margin_box.x, _current_y))
         return ((self.width - self.margin_box.x) / squares[0],
                 (self.height - self.margin_box.y) / squares[1])
 
@@ -938,7 +938,7 @@ class Graph(PublicApi):
         """Check if there is any data"""
         return any([
             len([
-                v for a in (s[1] if is_list_like(s) else [s])
+                v for a in (s[0] if is_list_like(s) else [s])
                 for v in (a if is_list_like(a) else [a])
                 if v is not None])
             for s in self.raw_series
